app.service('PpUtils', function(SettingsFactory, PpItem, DsUtils) {
  function _getMonday(d) {
    d = new Date(d);
    var day = d.getDay(),
    diff = d.getDate() - day + (day == 0 ? -6:1); // adjust when day is sunday
    return new Date(d.setDate(diff));
  }

  function _calcDailyAllowedPoints() {
    var age = parseFloat(SettingsFactory.getProfile().age);
    var poids = parseFloat(SettingsFactory.getProfile().weight);
    var taille = parseFloat(SettingsFactory.getProfile().height);

    if (SettingsFactory.getProfile().unit === 'imperial') {
      poids = poids * 0.45359237;
      taille = taille * 30.48;
    }

    if (age == NaN || taille == NaN || poids == NaN ) {
      return NaN;
    }

    var teem = 0.0;
    if (SettingsFactory.getProfile().sex === 'female') { // FEMALE
      teem = 387.0 - (7.31 * age) + 1.14 * ((10.9 * poids) + (6.607 * taille));
    } else {
      teem = 864.0 - (9.72 * age) + 1.12 * ((14.2 * poids) + (5.030 * taille));
    }

    var ateem = 0.9 * teem + 200;
    var target = Math.min(Math.max(Math.round((Math.max(ateem - 1000, 1000)) / 35) - 7 - 4, 26), 71);
    return target;
  }

  function _calcDailyConsumedPointsPromise() {
    var dateStart = _getMonday(new Date());dateStart.setHours(0,0,0,0);
    var dateEnd = new Date();dateEnd.setHours(23,59,59,999);
    var prom = PpItem.findAll({
      where: {
        date: {
          '<=': dateEnd,
          '>=': dateStart
        }
      },
      orderBy : [['date','DESC']]
    },{bypassCache:true}).then(function(result){
      var dailyConsumed = 0;
      var bonusConsumed = 0;
      var arrayDate= [];
      var dailyAllowed = _calcDailyAllowedPoints();
      result.forEach(function(item) {
        if (item.date.getDate() == dateEnd.getDate()) {
          if (item.pp>=0) {
            dailyConsumed += item.pp;
          }else {
            bonusConsumed += item.pp
          }  
        }

        if (arrayDate[item.date.getDate()] == undefined) {
          arrayDate[item.date.getDate()]=0;
        }
        arrayDate[item.date.getDate()]+=item.pp;

      });
      arrayDate.forEach(function(cur){
        bonusConsumed += Math.max(0, cur-dailyAllowed)
      });
      return {dailyConsumed:dailyConsumed,bonusConsumed:bonusConsumed};
    });
    return prom;
  }

  function _loadConsumptionPromise(start, limit) {
    var prom = PpItem.findAll({
      orderBy : [['date','DESC']],
      offset: start,
      limit: limit
    },{bypassCache:true});
    return prom;
  }

  function _saveActivity(activity) {
    var poids = parseFloat(SettingsFactory.getProfile().weight);
    var taille = parseFloat(SettingsFactory.getProfile().height);

    if (SettingsFactory.getProfile().unit === 'imperial') {
      poids = poids * 0.45359237;
      taille = taille * 30.48;
    }
    var actPts = 0;


    if (activity.level == 'low') {var factor = 0.000514117;}
    else if (activity.level == 'mod') {var factor = 0.000720911;}
    else if (activity.level == 'high') {var factor = 0.001780672;}
    else {factor = 0.0}
    var actPts = Math.round(poids * parseInt(activity.duration) * factor);

    var name = activity.name;
    if (name == undefined || name == "") name="--";
    DsUtils.createConso({name:name, pp:-actPts, date:new Date()});
  }

  return {
    calcDailyAllowedPoints : _calcDailyAllowedPoints,
    calcDailyConsumedPointsPromise : _calcDailyConsumedPointsPromise,
    saveActivity : _saveActivity,
    loadConsumptionPromise: _loadConsumptionPromise
  }
});
