app.service('importService', function($rootScope, $q, SettingsFactory, DsUtils, PpItem, $ionicLoading, $cordovaFile) {
  function _getSharedPreferences() {
    var deferred = $q.defer();
    sharedpreferences.getSharedPreferences('PREFS', 'MODE_PRIVATE',
      function(){
        deferred.resolve();
      }, function(err){
        deferred.reject('getSharedPreferences error : ' + err);
      });
    return deferred.promise;
  }
  function _getFloat(valToGet) {
    var deferred = $q.defer();
    sharedpreferences.getFloat(valToGet, function(val){
        deferred.resolve(val);
      }, function(err){
        deferred.reject('getFloat error : ' + err);
      });
    return deferred.promise;
  }
  function _getInt(valToGet) {
    var deferred = $q.defer();
    sharedpreferences.getInt(valToGet, function(val){
        deferred.resolve(val);
      }, function(err){
        deferred.reject('getInt error : ' + err);
      });
    return deferred.promise;
  }

  var toType = function(obj) {
    return ({}).toString.call(obj).match(/\s([a-zA-Z]+)/)[1].toLowerCase()
  }

  function _importProfile() {
    var profile = {unit:'metric'};
    return _getSharedPreferences().then(function() {
      return _getFloat('CAPITAL_POIDS');
    }).then(function(val) {
      profile.weight=parseFloat(val);
      return _getFloat('CAPITAL_TAILLE');
    }).then(function(val) {
      profile.height=parseFloat(val);
      return _getFloat('CAPITAL_AGE');
    }).then(function(val) {
      profile.age=parseFloat(val);
      return _getInt('CAPITAL_SEX');
    }).then(function(val){
      profile.sex= parseInt(val)==0?'female':'male';
      SettingsFactory.setProfile(profile);
    }).then(function(){
      return $cordovaFile.checkFile(cordova.file.dataDirectory, "ppConsumption")
    }).then(function(success){
      console.log("Finish  _importProfile: " + success);
      $ionicLoading.hide();
    }, function(err){
      localStorage["imported"] = angular.toJson({finish:true});
      console.log("Error!! ->" + err);
    });
  }

  function _importConsumtion() {
    return $cordovaFile.checkFile(cordova.file.dataDirectory, "ppConsumption")
    .then(function(){return PpItem.destroyAll()})
    .then(function(sucess){
      return $cordovaFile.readAsText(cordova.file.dataDirectory, "ppConsumption");
    }).then(function(data) {
      console.log("Prepare to split");
      var promise = [];
      data.split("\n").forEach(function(line){
        lineContent = line.split("!");
        if (lineContent.length!=3) return;
        var conso = {name:lineContent[0], date:new Date(lineContent[1]), pp:parseFloat(lineContent[2])}
        console.log(JSON.stringify(conso));
        promise.push(DsUtils.createConsoPromise(conso));
      });
      return $q.all(promise);
    }).then(function(success){
      console.log("importConsumtion finish");
    }, function(err){
      localStorage["imported"] = angular.toJson({finish:true});
      console.log("Error!! ->" + err);
    });
  }

  function _importAll() {
    var imported = localStorage["imported"];
    if (imported) {
      console.log("f"+angular.fromJson(imported));
      if (angular.fromJson(imported).finish) {
        return;
      }
    }

    if (localStorage.getItem("imported") == "true") return;
    $ionicLoading.show({
      template: 'Import...'
    });
    $q.all([_importConsumtion(),_importProfile()]).then(function(){
      console.log("all finish");
      $rootScope.$broadcast(EVT_UPDATE_CONSUMPTION);
      localStorage["imported"] = angular.toJson({finish:true});
      $ionicLoading.hide();
    }, function(err){
      console.log(err);
      localStorage["imported"] = angular.toJson({finish:true});
      $rootScope.$broadcast(EVT_UPDATE_CONSUMPTION);
      $ionicLoading.hide();
    });


  }
  return {
    importAll : _importAll
  }
});
